require 'test_helper'
require 'generators/express_settings/install/install_generator'

module ExpressSettings
  class InstallGeneratorTest < Rails::Generators::TestCase
    include GeneratorHelper
    tests InstallGenerator
    destination Rails.root.join('tmp', 'generators')
    setup :prepare_destination
    setup :copy_engine
    setup :copy_routes

    test "generates a migration to create the settings table and the settings model" do
      run_generator

      # Model and Migration
      assert_file 'app/models/generators/setting.rb'
      assert_migration 'db/migrate/create_generators_settings.rb'

      # Controller
      assert_file 'app/controllers/generators/admin/settings_controller.rb' do |content|
        assert_instance_method :show, content do |m|
          assert_match /@setting = Generators\.settings/, m
        end

        assert_instance_method :update, content do |m|
          assert_match /@setting = Generators\.settings/, m
          assert_match /@setting\.update_attributes\(setting_params\)/, m
        end

        assert_instance_method :setting_params, content do |m|
          assert_match /params\.require\(:setting\)\.permit!/, m
        end
      end

      # View
      assert_file 'app/views/generators/admin/settings/show.html.et' do |content|
        assert_match /h1 { "Generators Settings" }/, content
      end

      # Engine
      assert_file 'lib/generators/engine.rb' do |content|
        assert_equal <<-RUBY.strip_heredoc, content
          require 'express_settings'

          module Generators

            def self.settings
              Generators::Setting.first_or_create
            end

            class Engine < ::Rails::Engine
              isolate_namespace Generators
            end
          end
        RUBY
      end

      # Routes
      assert_file "config/routes.rb" do |content|
        assert_match(/resource :setting, only: \[:show, :update\]/, content)
      end
    end
  end
end
